/* vim: set sw=8 ts=8 si et: */
/*********************************************
* UART interface without interrupt
* Author: Guido Socher, Copyright: GPL 
* Copyright: GPL
**********************************************/
//#include <avr/signal.h>
#include <avr/interrupt.h>
#include <string.h>
#include <avr/io.h>
#include "uart.h"
#include "avr_compat.h"
#include "timeout.h"

void uart_init(unsigned int baud) 
{
        // baud=1=115.2K only with an external 3.6864MHz crystal:
        //unsigned int baud=64;  
        UBRR0H=(unsigned char) (baud >>8);
        UBRR0L=(unsigned char) (baud & 0xFF);
        /* enable tx/rx and no interrupt on tx/rx */
        UCSR0B =  (1<<RXEN0) | (1<<TXEN0);
        /* format: asynchronous, 8data, no parity, 1stop bit */
        UCSR0C = /*(1<<URSEL)|*/(3<<UCSZ00);
}

/* send one character to the rs232 */
void uart_sendchar(char c) 
{
        /* wait for empty transmit buffer */
        while (!(UCSR0A & (1<<UDRE0)));
        UDR0=c;
}
/* send string to the rs232 */
void uart_sendstr(char *s) 
{
        while (*s){
                uart_sendchar(*s);
                s++;
        }
}

void uart_sendstr_p(const prog_char *progmem_s)
/* print string from program memory on rs232 */
{
        char c;
        while ((c = pgm_read_byte(progmem_s++))) {
                uart_sendchar(c);
        }

}

/* get a byte from rs232
* this function does a blocking read */
unsigned char uart_getchar(unsigned char kickwd)  
{
        while(!(UCSR0A & (1<<RXC0))){
                // we can not aford a watchdog timeout 
                // because this is a blocking function
                if (kickwd){
                        wd_kick();
                }
        }
        return(UDR0);
}

/* read and discard any data in the receive buffer */
void uart_flushRXbuf(void)  
{
        unsigned char tmp;
        while(UCSR0A & (1<<RXC0)){
                tmp=UDR0;
        }
}

